==> fap-0.8.1.tar.bz2: fap-0.8.1/src/singleApp.cpp <==
/*
 *  Copyright (C) 2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "singleApp.h"

#include "utils.h"

#ifdef _WIN32
#  include <windows.h>
#else
#  include <fcntl.h>
#endif

FXIMPLEMENT(singleApp, FXApp, 0, 0)

singleApp::singleApp(const FXString &name, const FXString &vendor) :
	FXApp(name, vendor),
	_lockFileName(FXStringFormat("%s%c.fap.lock", FXSystem::getHomeDirectory().text(), PATHSEP)),
#ifdef _WIN32
	_fifoFileName("\\\\.\\pipe\\fap.fifo")
#else
	_fifoFileName(FXStringFormat("%s%c.fap.fifo", FXSystem::getHomeDirectory().text(), PATHSEP))
#endif
	{}

singleApp::~singleApp() {
#ifdef _WIN32
	CloseHandle(_overlapped.hEvent);
#endif
	if (_lockFile.isOpen()) {
		_lockFile.close();
		FXFile::remove(_lockFileName);
	}

	if (_fifoFile.isOpen()) {
		_fifoFile.close();
		FXFile::remove(_fifoFileName);
	}
}

bool singleApp::createPipe() {

#ifdef _WIN32
#ifdef UNICODE
	const FXnchar* pipeName=convertUtf(_fifoFileName);
#else
	const FXchar* pipeName=_fifoFileName.text();
#endif // UNICODE

	HANDLE pipe=CreateNamedPipe(
			pipeName,
			PIPE_ACCESS_DUPLEX|FILE_FLAG_OVERLAPPED, // PIPE_ACCESS_INBOUND
			PIPE_TYPE_MESSAGE|PIPE_READMODE_MESSAGE|PIPE_WAIT,
			1, 1024, 1024, 0, NULL);

	delete [] pipeName;

	if (pipe!=INVALID_HANDLE_VALUE) {
		FXTRACE((10, "Pipe created\n"));
		memset(&_overlapped, 0, sizeof(OVERLAPPED));
		_overlapped.hEvent=CreateEvent(NULL, TRUE, FALSE, NULL);
		if (_overlapped.hEvent!=INVALID_HANDLE_VALUE) {
			FXTRACE((10, "Event created\n"));
		}

		_fifoFile.open(pipe, FXIO::Reading);
		ConnectNamedPipe(_fifoFile.handle(), &_overlapped);
		if (GetLastError()==ERROR_IO_PENDING) {
			FXTRACE((10, "Waiting for connection\n"));
		}
		return true;
	}
#else
	FXPipe::create(_fifoFileName, FXIO::OwnerReadWrite);

	if (_fifoFile.open(_fifoFileName, FXIO::ReadWrite|FXIO::NonBlocking))
		return true;
#endif
	return false;
}

bool singleApp::init(int &argc, char **argv) {

	if (_lockFile.open(_lockFileName, FXIO::ReadWrite|FXIO::Create)
		&& !_lockFile.lock(0, 1)) {
		fxmessage("Already running\n");
		_lockFile.close();
		if (argc>1) {
#if defined (_WIN32) && defined (UNICODE)
			int narg;
			FXnchar** arg=CommandLineToArgvW(GetCommandLineW(), &narg);
			const FXchar* argv=convertNc(arg[1]);
			const FXString message=FXPath::absolute(argv);
			delete [] argv;
#else
			const FXString message=FXPath::absolute(argv[1]);
#endif
			FXTRACE((10, "Preparing message\n"));
			if (_fifoFile.open(_fifoFileName, FXIO::Writing)) {
				FXTRACE((10, "Sending message\n"));
				if (_fifoFile.writeBlock(message.text(), message.length())>0) {
					FXTRACE((10, "Message sent\n"));
				}
				_fifoFile.close();
			}
#ifdef _WIN32
			/*else {
				char message[512];
				FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(), NULL, message, 512, NULL);
				FXTRACE((10, "GetLastError: %s\n", message));
			}*/
#endif
		}
		return false;
	} else
		createPipe();

	FXApp::init(argc, argv);
	return true;
}

void singleApp::mapInput(FXObject *tgt, FXSelector sel) {

	if (_fifoFile.isOpen())
#ifdef _WIN32
		addInput(_overlapped.hEvent, INPUT_READ, tgt, sel);
#else
		addInput(_fifoFile.handle(), INPUT_READ, tgt, sel);
#endif
}

==> fap_0.8.8.orig.tar.bz2: fap-0.8.8/src/singleApp.cpp <==
/*
 *  Copyright (C) 2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "singleApp.h"

#include "utils.h"

#ifdef _WIN32
#  include <windows.h>
#else
#  include <fcntl.h>
#endif

FXIMPLEMENT(singleApp, FXApp, 0, 0)

singleApp::singleApp(const FXString &name, const FXString &vendor) :
	FXApp(name, vendor),
	_lockFileName(FXStringFormat("%s%c.fap.lock", getRuntimeDir().text(), PATHSEP)),
#ifdef _WIN32
	_fifoFileName("\\\\.\\pipe\\fap.fifo")
#else
	_fifoFileName(FXStringFormat("%s%c.fap.fifo", getRuntimeDir().text(), PATHSEP))
#endif
	{}

singleApp::~singleApp() {
#ifdef _WIN32
	CloseHandle(_overlapped.hEvent);
#endif
	if (_lockFile.isOpen()) {
		_lockFile.close();
		FXFile::remove(_lockFileName);
	}

	if (_fifoFile.isOpen()) {
		_fifoFile.close();
		FXFile::remove(_fifoFileName);
	}
}

bool singleApp::createPipe() {

#ifdef _WIN32
#ifdef UNICODE
	const FXnchar* pipeName=convertUtf(_fifoFileName);
#else
	const FXchar* pipeName=_fifoFileName.text();
#endif // UNICODE

	HANDLE pipe=CreateNamedPipe(
			pipeName,
			PIPE_ACCESS_DUPLEX|FILE_FLAG_OVERLAPPED, // PIPE_ACCESS_INBOUND
			PIPE_TYPE_MESSAGE|PIPE_READMODE_MESSAGE|PIPE_WAIT,
			1, 1024, 1024, 0, NULL);

	delete [] pipeName;

	if (pipe!=INVALID_HANDLE_VALUE) {
		FXTRACE((10, "Pipe created\n"));
		memset(&_overlapped, 0, sizeof(OVERLAPPED));
		_overlapped.hEvent=CreateEvent(NULL, TRUE, FALSE, NULL);
		if (_overlapped.hEvent!=INVALID_HANDLE_VALUE) {
			FXTRACE((10, "Event created\n"));
		}

		_fifoFile.open(pipe, FXIO::Reading);
		ConnectNamedPipe(_fifoFile.handle(), &_overlapped);
		if (GetLastError()==ERROR_IO_PENDING) {
			FXTRACE((10, "Waiting for connection\n"));
		}
		return true;
	}
#else
	FXPipe::create(_fifoFileName, FXIO::OwnerReadWrite);

	if (_fifoFile.open(_fifoFileName, FXIO::ReadWrite|FXIO::NonBlocking))
		return true;
#endif
	return false;
}

bool singleApp::init(int &argc, char **argv) {

	if (_lockFile.open(_lockFileName, FXIO::ReadWrite|FXIO::Create)
		&& !_lockFile.lock(0, 1)) {
		fxmessage("Already running\n");
		_lockFile.close();
		if (argc>1) {
#if defined (_WIN32) && defined (UNICODE)
			int narg;
			FXnchar** arg=CommandLineToArgvW(GetCommandLineW(), &narg);
			const FXchar* argv=convertNc(arg[1]);
			const FXString message=FXPath::absolute(argv);
			delete [] argv;
#else
			const FXString message=FXPath::absolute(argv[1]);
#endif
			FXTRACE((10, "Preparing message\n"));
			if (_fifoFile.open(_fifoFileName, FXIO::Writing)) {
				FXTRACE((10, "Sending message\n"));
				if (_fifoFile.writeBlock(message.text(), message.length())>0) {
					FXTRACE((10, "Message sent\n"));
				}
				_fifoFile.close();
			}
#ifdef _WIN32
			/*else {
				char message[512];
				FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(), NULL, message, 512, NULL);
				FXTRACE((10, "GetLastError: %s\n", message));
			}*/
#endif
		}
		return false;
	} else
		createPipe();

	FXApp::init(argc, argv);
	return true;
}

void singleApp::mapInput(FXObject *tgt, FXSelector sel) {

	if (_fifoFile.isOpen())
#ifdef _WIN32
		addInput(_overlapped.hEvent, INPUT_READ, tgt, sel);
#else
		addInput(_fifoFile.handle(), INPUT_READ, tgt, sel);
#endif
}
